@@ -29,6 +29,7 @@
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+
 #include "catalog.h"
 #include "typedefs.h"
 #include "file-utils.h"
@@ -36,134 +37,12 @@
 #include "gthumb-error.h"
 
 
-#define SEARCH_HEADER "# Search\n"
 #define SORT_FIELD "# sort: "
 #define MAX_LINE_LENGTH 4096
 
 static char *sort_names[] = { "none", "name", "path", "size", "time", "manual" };
 
 
-char *
-get_catalog_full_path (const char *relative_path)
-{
-	char *path;
-	char *separator;
-
-	/* Do not allow .. in the relative_path otherwise the user can go
-	 * to any directory, while he shouldn't exit from RC_CATALOG_DIR. */
-	if ((relative_path != NULL) && (strstr (relative_path, "..") != NULL))
-		return NULL;
-
-	if (relative_path == NULL)
-		separator = NULL;
-	else
-		separator = (relative_path[0] == '/') ? "" : "/";
-
-	path = g_strconcat (g_get_home_dir (),
-			    "/",
-			    RC_CATALOG_DIR,
-			    separator,
-			    relative_path,
-			    NULL);
-	return path;
-}
-
-
-gboolean
-delete_catalog_dir (const char  *full_path, 
-		    gboolean     recursive,
-		    GError     **gerror)
-{
-	if (dir_remove (full_path))
-		return TRUE;
-
-	if (gerror != NULL) {
-		const char *rel_path;
-		char       *base_path;
-		char       *utf8_path;
-		const char *details;
-
-		base_path = get_catalog_full_path (NULL);
-		rel_path = full_path + strlen (base_path) + 1;
-		g_free (base_path);
-
-		utf8_path = g_filename_display_name (rel_path);
-
-		switch (gnome_vfs_result_from_errno ()) {
-		case GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY:
-			details = _("Library not empty");
-			break;
-		default:
-			details = errno_to_string ();
-			break;
-		}
-
-		*gerror = g_error_new (GTHUMB_ERROR,
-				       errno,
-				       _("Cannot remove library \"%s\": %s"),
-				       utf8_path,
-				       details);
-		g_free (utf8_path);
-	}
-
-	return FALSE;
-}
-
-
-gboolean
-delete_catalog (const char  *full_path,
-		GError     **gerror)
-{
-	if (! file_unlink (full_path)) {
-		if (gerror != NULL) {
-			const char *rel_path;
-			char       *base_path;
-			char       *catalog;			
-
-			base_path = get_catalog_full_path (NULL);
-			rel_path = full_path + strlen (base_path) + 1;
-			g_free (base_path);
-			catalog = remove_extension_from_path (rel_path);
-
-			*gerror = g_error_new (GTHUMB_ERROR,
-					       errno,
-					       _("Cannot remove catalog \"%s\": %s"),
-					       catalog,
-					       errno_to_string ());
-			g_free (catalog);
-		}
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-
-gboolean
-file_is_search_result (const char *fullpath)
-{
-	GnomeVFSResult  r;
-	char           *epath;
-	GnomeVFSHandle *handle;
-	char            line[50] = "";
-
-	epath = escape_uri (fullpath);
-	r = gnome_vfs_open (&handle, epath, GNOME_VFS_OPEN_READ);
-	g_free (epath);
-
-	if (r != GNOME_VFS_OK)
-		return FALSE;
-
-	r = gnome_vfs_read (handle, line, strlen (SEARCH_HEADER), NULL);
-	gnome_vfs_close (handle);
-
-	if ((r != GNOME_VFS_OK) || (line[0] == 0))
-		return FALSE;
-
-	return strncmp (line, SEARCH_HEADER, strlen (SEARCH_HEADER)) == 0;
-}
-
-
 Catalog*  
 catalog_new ()
 {
@@ -235,30 +114,32 @@ catalog_is_search_result (Catalog *catalog)
 static void
 copy_unquoted (char *unquoted, char *line)
 {
-	gint l = strlen (line);
+	int len = strlen (line);
 
-	strncpy (unquoted, line + 1, l - 3);
-	unquoted[l - 3] = 0;
+	strncpy (unquoted, line + 1, len - 2);
+	unquoted[len - 2] = 0;
 }
 
 
 gboolean
-catalog_load_from_disk (Catalog     *catalog,
-			const char  *fullpath,
-			GError     **gerror)
+catalog_load_from_disk__common (Catalog     *catalog,
+				const char  *uri,
+				GError     **gerror,
+				gboolean     load_file_list)
 {
-	FILE     *f;
-	char      line[MAX_LINE_LENGTH];
-	gboolean  file_list = FALSE;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+	char            line[MAX_LINE_LENGTH];
+	gboolean        file_list = FALSE;
 
-	f = fopen (fullpath, "r");
-	if (f == NULL) {
+	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
+	if (result != GNOME_VFS_OK) {
 		if (gerror != NULL)
 			*gerror = g_error_new (GTHUMB_ERROR,
-					       errno,
+					       result,
 					       _("Cannot open catalog \"%s\": %s"),
-					       fullpath,
-					       errno_to_string ());
+					       uri,
+					       gnome_vfs_result_to_string (result));
 		return FALSE;
 	}
 
@@ -269,7 +150,7 @@ catalog_load_from_disk (Catalog     *catalog,
 	if (catalog->search_data != NULL)
 		search_data_free (catalog->search_data);
 
-	catalog->path = g_strdup (fullpath);
+	catalog->path = g_strdup (uri);
 	catalog->list = NULL;
 	catalog->search_data = NULL;
 
@@ -304,7 +185,10 @@ catalog_load_from_disk (Catalog     *catalog,
 	 *    "filename_n"
 	 *   
 	 */
-	while (fgets (line, sizeof (line), f)) {
+	while ((result = _gnome_vfs_read_line (handle, 
+					       line, 
+					       MAX_LINE_LENGTH, 
+					       NULL)) == GNOME_VFS_OK) {
 		char *file_name;
 
 		if (*line == 0)
@@ -324,41 +208,41 @@ catalog_load_from_disk (Catalog     *catalog,
 
 			/* * start from */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			copy_unquoted (unquoted, line);
 			search_data_set_start_from (catalog->search_data,
 						    unquoted);
 
 			/* * recursive */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			copy_unquoted (unquoted, line);
 			search_data_set_recursive (catalog->search_data, strcmp (unquoted, "TRUE") == 0);
 
 			/* * file pattern */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			copy_unquoted (unquoted, line);
 			search_data_set_file_pattern (catalog->search_data,
 						      unquoted);
 
 			/* * comment pattern */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			copy_unquoted (unquoted, line);
 			search_data_set_comment_pattern (catalog->search_data,
 							 unquoted);
 
 			/* * place pattern */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			copy_unquoted (unquoted, line);
 			search_data_set_place_pattern (catalog->search_data,
 						       unquoted);
 
 			/* * keywords pattern */
 
-			fgets (line, sizeof (line), f);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
 			if (*line != '"') {
 				line_ofs = 1;
 				/* * all keywords */
@@ -371,12 +255,14 @@ catalog_load_from_disk (Catalog     *catalog,
 
 			/* * date */
 
-			fscanf (f, "%ld\n", &date);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
+			sscanf (line, "%ld", &date);
 			search_data_set_date (catalog->search_data, date);
 
 			/* * date scope */
 
-			fscanf (f, "%d\n", &date_scope);
+			_gnome_vfs_read_line (handle, line, sizeof (line), NULL);
+			sscanf (line, "%d", &date_scope);
 			search_data_set_date_scope (catalog->search_data, 
 						    date_scope);
 
@@ -389,7 +275,7 @@ catalog_load_from_disk (Catalog     *catalog,
 			int            i;
 
 			sort_type = line + strlen (SORT_FIELD);
-			sort_type[strlen (sort_type) - 1] = 0;
+			sort_type[strlen (sort_type)] = 0;
 
 			for (i = 0; i < G_N_ELEMENTS (sort_names); i++)
 				if (strcmp (sort_type, sort_names[i]) == 0) {
@@ -402,11 +288,15 @@ catalog_load_from_disk (Catalog     *catalog,
 			continue;
 		}
 
+		if (!load_file_list)
+			break;
+
 		file_list  = TRUE;
-		file_name = g_strndup (line + 1, strlen (line) - 3);
+		file_name = g_strndup (line + 1, strlen (line) - 2);
 		catalog->list = g_list_prepend (catalog->list, file_name);
 	}
-	fclose (f);
+
+	gnome_vfs_close (handle);
 
 	catalog->list = g_list_reverse (catalog->list);
 
@@ -414,16 +304,37 @@ catalog_load_from_disk (Catalog     *catalog,
 }
 
 
-static void
-error_on_saving (char    *path, 
-		 GError **gerror)
+gboolean
+catalog_load_from_disk (Catalog     *catalog,
+			const char  *uri,
+			GError     **gerror)
 {
+	return catalog_load_from_disk__common (catalog, uri, gerror, TRUE);
+}
+
+
+gboolean
+catalog_load_search_data_from_disk (Catalog     *catalog,
+				    const char  *uri,
+				    GError     **gerror)
+{
+	return catalog_load_from_disk__common (catalog, uri, gerror, FALSE);
+}
+
+
+static gboolean
+error_on_saving (GnomeVFSHandle  *handle,
+		 char            *path, 
+		 GError         **gerror)
+{
+	gnome_vfs_close (handle);
 	if (gerror != NULL)
 		*gerror = g_error_new (GTHUMB_ERROR,
 				       errno,
 				       _("Cannot save catalog \"%s\": %s"),
 				       path,
 				       errno_to_string ());
+	return FALSE;
 }
 
 
@@ -431,22 +342,21 @@ gboolean
 catalog_write_to_disk (Catalog     *catalog,
 		       GError     **gerror)
 {
-	FILE  *f;
-	gchar *path;
-	GList *scan;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+	GList          *scan;
 
 	g_return_val_if_fail (catalog != NULL, FALSE);
 	g_return_val_if_fail (catalog->path != NULL, FALSE);
 
-	path = catalog->path;
-	f = fopen (path, "w");
-	if (!f)	{
+	result = gnome_vfs_open (&handle, catalog->path, GNOME_VFS_OPEN_WRITE);
+	if (result != GNOME_VFS_OK) {
 		if (gerror != NULL)
 			*gerror = g_error_new (GTHUMB_ERROR,
-					       errno,
+					       result,
 					       _("Cannot open catalog \"%s\": %s"),
-					       path,
-					       errno_to_string ());
+					       catalog->path,
+					       gnome_vfs_result_to_string (result));
 		return FALSE;
 	}
 
@@ -454,83 +364,68 @@ catalog_write_to_disk (Catalog     *catalog,
 		SearchData *search_data = catalog->search_data;
 
 		/* write search data. */
+		if (_gnome_vfs_write_line (handle, SEARCH_HEADER) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, SEARCH_HEADER)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "\"%s\"", 
+					   search_data->start_from) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", search_data->start_from)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "\"%s\"", 
+					   (search_data->recursive ? "TRUE" : "FALSE")) != GNOME_VFS_OK)
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", (search_data->recursive ? "TRUE" : "FALSE"))) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "\"%s\"", 
+					   search_data->file_pattern) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", search_data->file_pattern)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "\"%s\"", 
+					   search_data->comment_pattern) != GNOME_VFS_OK)
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", search_data->comment_pattern)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "\"%s\"", 
+					   search_data->place_pattern) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", search_data->place_pattern)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "%d\"%s\"", 
+					   catalog->search_data->all_keywords,
+					   search_data->keywords_pattern) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "%d", catalog->search_data->all_keywords)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "%ld", 
+					   search_data->date) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 
-		if (! fprintf (f, "\"%s\"\n", search_data->keywords_pattern)) {
-			fclose (f);
-			return FALSE;
-		}
-
-		if (! fprintf (f, "%ld\n", search_data->date)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
-
-		if (! fprintf (f, "%d\n", catalog->search_data->date_scope)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle, 
+					   "%d", 
+					   catalog->search_data->date_scope) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 	}
 
 	/* sort method */
 
-	fprintf (f, "%s%s\n", SORT_FIELD, sort_names[catalog->sort_method]);
+	if (_gnome_vfs_write_line (handle, 
+				   "%s%s", 
+				   SORT_FIELD, 
+				   sort_names[catalog->sort_method]) != GNOME_VFS_OK) 
+		return error_on_saving (handle, catalog->path, gerror);
 
 	/* write the file list. */
 
 	for (scan = catalog->list; scan; scan = scan->next) 
-		if (! fprintf (f, "\"%s\"\n", (gchar*) scan->data)) {
-			fclose (f);
-			error_on_saving (path, gerror);
-			return FALSE;
-		}
+		if (_gnome_vfs_write_line (handle,
+					   "\"%s\"", 
+					   (char*) scan->data) != GNOME_VFS_OK) 
+			return error_on_saving (handle, catalog->path, gerror);
 	
-	fclose (f);
-
-	return TRUE;
+	return gnome_vfs_close (handle) == GNOME_VFS_OK;
 }
 
 
