@@ -97,7 +97,6 @@ __save_image (GtkWindow       *parent,
 	gboolean  file_exists;
 	gboolean  image_saved = FALSE;
 
-
 	if (filename == NULL)
 		return FALSE;
 
@@ -145,10 +144,10 @@ __save_image (GtkWindow       *parent,
 	if ((mime_type != NULL)
 	    && is_mime_type_writable (mime_type)) {
 		GError      *error = NULL;
-		const char  *image_type = mime_type + 6;
+		const char  *image_type = mime_type + strlen("image/");
 		char       **keys = NULL;
 		char       **values = NULL;
-		
+
 		if (dlg_save_options (parent,
 				      image_type,
 				      &keys,
@@ -205,20 +204,9 @@ file_save_ok_cb (GtkDialog *file_sel,
 
 	opt_menu = g_object_get_data (G_OBJECT (file_sel), "opt_menu");
 	idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_menu));
-	if (idx == IMAGE_TYPE_AUTOMATIC) {
-		char *n1 = g_filename_to_utf8 (filename, -1, 0, 0, 0);
-		if (n1 != NULL) {
-			char *n2, *n3;
-			n2 = g_utf8_strdown (n1, -1);
-			n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
-			if (n3 != NULL)
-				mime_type = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
-			g_free (n3);
-			g_free (n2);
-			g_free (n1);
-		}
-
-	} else
+	if (idx == IMAGE_TYPE_AUTOMATIC) 
+		mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+	else
 		mime_type = mime_types [idx - 2];
 
 	__save_image (parent, filename, mime_type, pixbuf, data, file_sel); 
@@ -371,7 +359,7 @@ dlg_save_image (GtkWindow       *parent,
 	data->done_func = done_func;
 	data->done_data = done_data;
 
-	mime_type = gnome_vfs_mime_type_from_name_or_default (filename, NULL);
+	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
 
 	__save_image (parent, filename, mime_type, pixbuf, data, NULL);
 }
