@@ -38,7 +38,6 @@ static struct {
 	{ GTHUMB_STOCK_BRIGHTNESS_CONTRAST, brightness_contrast_22_rgba, brightness_contrast_16_rgba },
 	{ GTHUMB_STOCK_CAMERA,              camera_24_rgba,              NULL },
 	{ GTHUMB_STOCK_CATALOG,             catalog_24_rgba,             catalog_16_rgba },
-	{ GTHUMB_STOCK_CDROM,               cdrom_24_rgba,               NULL },
 	{ GTHUMB_STOCK_CHANGE_DATE,         change_date_16_rgba,         NULL },
 	{ GTHUMB_STOCK_COLOR_BALANCE,       color_balance_22_rgba,       color_balance_16_rgba },
 	{ GTHUMB_STOCK_CROP,                crop_16_rgba,                NULL },
@@ -59,7 +58,6 @@ static struct {
 	{ GTHUMB_STOCK_NORMAL_VIEW,         exit_fullscreen_24_rgba,     NULL },
 	{ GTHUMB_STOCK_POSTERIZE,           posterize_22_rgba,           posterize_16_rgba },
 	{ GTHUMB_STOCK_PREVIOUS_IMAGE,      prev_image_24_rgba,          NULL },
-	{ GTHUMB_STOCK_PROPERTIES,          image_info_24_rgba,          image_info_16_rgba },
 	{ GTHUMB_STOCK_REDUCE_COLORS,       reduce_colors_16_rgba,       NULL },
 	{ GTHUMB_STOCK_RESET,               reset_16_rgba,               NULL },
 	{ GTHUMB_STOCK_RESIZE,              resize_22_rgba,              resize_16_rgba },
@@ -68,7 +66,6 @@ static struct {
 	{ GTHUMB_STOCK_ROTATE_90_CC,        rotate_270_24_rgba,          rotate_270_16_rgba },
 	{ GTHUMB_STOCK_SEARCH,              catalog_search_16_rgba,      NULL },
 	{ GTHUMB_STOCK_SHOW_CATALOGS,       catalog_24_rgba,             catalog_16_rgba },
-	{ GTHUMB_STOCK_SHOW_FOLDERS,        dir_24_rgba,                 dir_16_rgba },
 	{ GTHUMB_STOCK_SHOW_IMAGE,          image_24_rgba,               NULL },
 	{ GTHUMB_STOCK_SLIDESHOW,           slideshow_24_rgba,           slideshow_16_rgba },
 	{ GTHUMB_STOCK_SWAP,                swap_24_rgba,                swap_16_rgba },
