@@ -23,11 +23,13 @@
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkrgb.h>
-#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs.h>
+
 #include "catalog-png-exporter.h"
 #include "comments.h"
 #include "file-utils.h"
@@ -128,7 +130,7 @@ image_data_new (gchar *filename)
 
 	idata = g_new (ImageData, 1);
 
-	cdata = comments_load_comment (filename);
+	cdata = comments_load_comment (filename, TRUE);
 	idata->comment = comments_get_comment_as_string (cdata, "\n", "\n");
 	if (cdata != NULL)
 		comment_data_free (cdata);
@@ -790,17 +792,17 @@ set_item_caption (CatalogPngExporter *ce,
 
 	if ((ce->caption_fields & GTH_CAPTION_FILE_PATH) 
 	    && (ce->caption_fields & GTH_CAPTION_FILE_NAME)) {
-		char *utf8_name = g_filename_to_utf8 (idata->filename, -1, 0, 0, 0);
+		char *utf8_name = g_filename_display_name (idata->filename);
 		idata->caption_row[row++] = utf8_name;
 	} else {
 		if (ce->caption_fields & GTH_CAPTION_FILE_PATH) {
 			char *path = remove_level_from_path (idata->filename);
-			char *utf8_name = g_filename_to_utf8 (path, -1, 0, 0, 0);
+			char *utf8_name = g_filename_display_name (path);
 			idata->caption_row[row++] = utf8_name;
 			g_free (path);
 		} else if (ce->caption_fields & GTH_CAPTION_FILE_NAME) {
 			const char *name = file_name_from_path (idata->filename);
-			char *utf8_name = g_filename_to_utf8 (name, -1, 0, 0, 0);
+			char *utf8_name = g_filename_display_name (name);
 			idata->caption_row[row++] = utf8_name;
 		}
 	}
@@ -1330,7 +1332,7 @@ load_next_file (CatalogPngExporter *ce)
 
 	g_free (ce->info);
 
-	utf8_name = g_filename_to_utf8 (file_name_from_path (filename), -1, 0, 0, 0);
+	utf8_name = g_filename_display_name (file_name_from_path (filename));
 	ce->info = g_strdup_printf (_("Loading image: %s"), utf8_name);
 	g_free (utf8_name);
 
@@ -1559,7 +1561,7 @@ begin_page (CatalogPngExporter *ce,
 	g_free (ce->info);
 
 	filename = _g_get_name_from_template (ce->templatev, ce->start_at + page_n - 1);
-	utf8_name = g_filename_to_utf8 (filename, -1, 0, 0, 0);
+	utf8_name = g_filename_display_name (filename);
 	ce->info = g_strdup_printf (_("Creating image: %s.%s"), 
 				    utf8_name, 
 				    ce->file_type);
@@ -1579,7 +1581,7 @@ begin_page (CatalogPngExporter *ce,
 					      ce->start_at + page_n - 1);
 	path = g_strconcat (ce->directory, "/", filename, ".html", NULL);
 
-	uri = gnome_vfs_uri_new (path);
+	uri = new_uri_from_path (path);
 	g_free (path);
 
 	if (uri == NULL) {
@@ -1776,7 +1778,7 @@ paint_image (CatalogPngExporter *ce,
 				    x, y,
 				    width,
 				    height);
-
+		
 		gdk_pixbuf_render_to_drawable_alpha (image,
 						     ce->pixmap, 
 						     0, 0,
@@ -1785,6 +1787,7 @@ paint_image (CatalogPngExporter *ce,
 						     GDK_PIXBUF_ALPHA_BILEVEL,
 						     112,
 						     GDK_RGB_DITHER_MAX, 0, 0);
+		
 	} else
 		gdk_pixbuf_render_to_drawable (image,
 					       ce->pixmap, 
