@@ -26,17 +26,18 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
 #include <glade/glade.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+
 #include "typedefs.h"
 #include "catalog.h"
 #include "catalog-list.h"
 #include "file-utils.h"
 #include "main.h"
-#include "gthumb-window.h"
+#include "gth-browser.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "gconf-utils.h"
 
@@ -51,8 +52,11 @@ typedef struct {
 	GtkWidget    *new_dir_btn;
 	GtkWidget    *ok_btn;
 	GtkWidget    *cancel_btn;
+	GtkWidget    *cat_view_dest_checkbutton;
 
-	GThumbWindow *window;
+	gboolean      can_view_catalog;
+
+	GthWindow    *window;
 	CatalogList  *cat_list;
 	char         *current_dir;
 
@@ -73,13 +77,14 @@ static void
 new_catalog_cb (GtkWidget *widget, 
 		gpointer   p)
 {
-	DialogData *data = p;
-	char       *name_utf8, *name;
-	char       *path;
-	int         fd;
-	GtkTreeIter iter;
+	DialogData     *data = p;
+	char           *name_utf8, *name;
+	char           *path;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+	GtkTreeIter     iter;
 
-	name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (data->window->app),
+	name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (data->window),
 					     GTK_DIALOG_MODAL,
 					     _("Enter the catalog name: "),
 					     "",
@@ -100,10 +105,13 @@ new_catalog_cb (GtkWidget *widget,
 			    NULL);
 	g_free (name);
 
-	fd = creat (path, 0644);
-	close (fd);
-
-	sync ();
+	result = gnome_vfs_create (&handle,
+				   path,
+				   GNOME_VFS_OPEN_WRITE,
+				   TRUE,
+				   0644);
+	if (result == GNOME_VFS_OK)
+		gnome_vfs_close (handle);
 
 	/* update the catalog list. */
 
@@ -130,7 +138,7 @@ new_dir_cb (GtkWidget *widget,
 	char *name;
 	char *path;
 
-	utf8_name = _gtk_request_dialog_run (GTK_WINDOW (data->window->app),
+	utf8_name = _gtk_request_dialog_run (GTK_WINDOW (data->window),
 					     GTK_DIALOG_MODAL,
 					     _("Enter the library name: "),
 					     "",
@@ -144,15 +152,15 @@ new_dir_cb (GtkWidget *widget,
 	name = g_filename_from_utf8 (utf8_name, -1, 0, 0, 0);
 	if (name == NULL)
 		return;
-
 	path = g_strconcat (data->current_dir,
 			    "/",
 			    name,
 			    NULL);
-	g_free (name);
 
-	mkdir (path, 0775);
+	dir_make (path, 0775);
+
 	g_free (path);
+	g_free (name);
 
 	/* update the catalog list. */
 	catalog_list_change_to (data->cat_list, data->current_dir);
@@ -183,18 +191,23 @@ add_to_catalog__ok_cb (GtkWidget *widget,
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
 		return;
 	}
-
 	for (scan = data->data.list; scan; scan = scan->next)
 		catalog_add_item (catalog, (char*) scan->data);
-
 	if (! catalog_write_to_disk (catalog, &gerror)) 
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
 	else
-		all_windows_notify_cat_files_added (cat_path, data->data.list);
+		all_windows_notify_cat_files_created (cat_path, data->data.list);
+
+	eel_gconf_set_path (PREF_ADD_TO_CATALOG_LAST_CATALOG, cat_path);
+	if (data->can_view_catalog) {
+		gboolean view;
+		view = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->cat_view_dest_checkbutton));
+		eel_gconf_set_boolean (PREF_ADD_TO_CATALOG_VIEW, view);
+		if (view)
+			gth_browser_go_to_catalog (GTH_BROWSER (data->window), cat_path);
+	}
 
 	catalog_free (catalog);
-	
-	eel_gconf_set_path (PREF_ADD_TO_CATALOG_LAST_CATALOG, cat_path);
 	g_free (cat_path);
 
 	gtk_widget_destroy (data->dialog);
@@ -257,8 +270,8 @@ add_to_catalog__destroy_cb (GtkWidget *widget,
 
 /* create the "add to catalog" dialog. */
 void
-dlg_add_to_catalog (GThumbWindow *window,
-		    GList        *list)
+dlg_add_to_catalog (GthWindow *window,
+		    GList     *list)
 {
 	GtkTreeSelection *selection;
 	DialogData       *data;
@@ -271,6 +284,7 @@ dlg_add_to_catalog (GThumbWindow *window,
 	data->window = window;
 	data->cat_list = catalog_list_new (FALSE);
 	data->data.list = list;
+	data->can_view_catalog = GTH_IS_BROWSER (window);
 
 	last_catalog = eel_gconf_get_path (PREF_ADD_TO_CATALOG_LAST_CATALOG, NULL);
 	if (path_is_file (last_catalog)) {
@@ -290,6 +304,7 @@ dlg_add_to_catalog (GThumbWindow *window,
 	list_hbox = glade_xml_get_widget (data->gui, "cat_list_hbox");
 	data->new_catalog_btn = glade_xml_get_widget (data->gui, "cat_new_catalog_btn");
 	data->new_dir_btn = glade_xml_get_widget (data->gui, "cat_new_dir_btn");
+	data->cat_view_dest_checkbutton = glade_xml_get_widget (data->gui, "cat_view_dest_checkbutton");
 	cat_catalogs_label = glade_xml_get_widget (data->gui, "cat_catalogs_label");
 	data->ok_btn = glade_xml_get_widget (data->gui, "cat_ok_btn");
 	data->cancel_btn = glade_xml_get_widget (data->gui, "cat_cancel_btn");
@@ -343,13 +358,20 @@ dlg_add_to_catalog (GThumbWindow *window,
 		g_free (last_catalog);
 	}
 
+	if (data->can_view_catalog) 
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->cat_view_dest_checkbutton),
+					      eel_gconf_get_boolean (PREF_ADD_TO_CATALOG_VIEW, FALSE));
+	else
+		gtk_widget_hide (data->cat_view_dest_checkbutton);
+	gtk_widget_show_all (data->cat_list->root_widget);
+
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
-				      GTK_WINDOW (window->app));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog),
+				      GTK_WINDOW (window));
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
-	gtk_widget_show_all (data->dialog);
+	gtk_widget_show (data->dialog);
 }
 
 
@@ -414,8 +436,8 @@ move_to_catalog_dir__ok_cb (GtkWidget  *widget,
 
 /* create the "move to catalog directory" dialog. */
 void
-dlg_move_to_catalog_directory (GThumbWindow *window,
-			       gchar *catalog_path)
+dlg_move_to_catalog_directory (GthWindow *window,
+			       gchar     *catalog_path)
 {
 	DialogData *data;
 	GtkWidget  *list_hbox;
@@ -425,6 +447,7 @@ dlg_move_to_catalog_directory (GThumbWindow *window,
 	data->window = window;
 	data->current_dir = get_catalog_full_path (NULL);
 	data->cat_list = catalog_list_new (FALSE);
+	catalog_list_show_dirs_only (data->cat_list, TRUE);
 	data->data.catalog_path = catalog_path;
 
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE, NULL, NULL);
@@ -474,13 +497,15 @@ dlg_move_to_catalog_directory (GThumbWindow *window,
 	catalog_list_change_to (data->cat_list, data->current_dir);
 	gtk_widget_set_sensitive (data->new_catalog_btn, FALSE);
 	gtk_window_set_title (GTK_WINDOW (data->dialog), _("Move Catalog to..."));
+	gtk_widget_hide (glade_xml_get_widget (data->gui, "cat_view_dest_checkbutton"));
+	gtk_widget_show_all (data->cat_list->root_widget);
 
 	/* run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
-				      GTK_WINDOW (window->app));
+				      GTK_WINDOW (window));
 	
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 
-	gtk_widget_show_all (data->dialog);
+	gtk_widget_show (data->dialog);
 }
