@@ -24,12 +24,11 @@
 #define DLG_CATALOG_H
 
 #include <glib.h>
-#include "gthumb-window.h"
+#include "gth-window.h"
 
-void   dlg_add_to_catalog                   (GThumbWindow *window,
-					     GList *list);
-
-void   dlg_move_to_catalog_directory        (GThumbWindow *window,
-					     gchar *catalog_path);
+void   dlg_add_to_catalog              (GthWindow *window,
+					GList     *list);
+void   dlg_move_to_catalog_directory   (GthWindow *window,
+					gchar     *catalog_path);
 
 #endif /* DLG_CATALOG_H */
