@@ -68,6 +68,7 @@
 #define GLADE_EXPORTER_FILE "gthumb_png_exporter.glade"
 #define DEFAULT_WIN_WIDTH 200
 #define DEFAULT_WIN_HEIGHT 400
+#define DEFAULT_COMMENT_PANE_SIZE 100
 #define DISPLAY_PROGRESS_DELAY 750
 #define PANE_MIN_SIZE 60
 #define GCONF_NOTIFICATIONS 3
@@ -462,7 +463,7 @@ gth_viewer_unrealize (GtkWidget *widget)
 	/* save ui preferences. */
 
 	save_window_size (viewer);
-	eel_gconf_set_integer (PREF_UI_COMMENT_PANE_SIZE, gtk_paned_get_position (GTK_PANED (priv->image_main_pane)));
+	eel_gconf_set_integer (PREF_UI_COMMENT_PANE_SIZE, _gtk_widget_get_height (widget) - gtk_paned_get_position (GTK_PANED (viewer->priv->image_main_pane)));
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
@@ -859,8 +860,8 @@ save_jpeg_data (GthViewer  *viewer,
 
 #ifdef HAVE_LIBIPTCDATA
 	if (priv->iptc_data != NULL) {
-		unsigned char *out_buf, *iptc_buf;
-		int            iptc_len, ps3_len;
+		guchar *out_buf, *iptc_buf;
+		guint   iptc_len, ps3_len;
 
 		out_buf = g_malloc (256*256);
 		iptc_data_save (priv->iptc_data, &iptc_buf, &iptc_len);
@@ -1094,7 +1095,7 @@ viewer_drag_data_get  (GtkWidget        *widget,
 	gtk_selection_data_set (selection_data,
 				selection_data->target,
 				8, 
-				path, strlen (path));
+				(guchar*)path, strlen (path));
 	g_free (path);
 }
 
@@ -1139,6 +1140,7 @@ viewer_key_press_cb (GtkWidget   *widget,
 		     gpointer     data)
 {
 	GthViewer   *viewer = data;
+	GthWindow   *window = (GthWindow*) viewer;
 	ImageViewer *image_viewer = (ImageViewer*)viewer->priv->viewer;
 	gboolean     retval = FALSE;
 
@@ -1191,30 +1193,35 @@ viewer_key_press_cb (GtkWidget   *widget,
 		retval = TRUE;
 		break;
 
-		/* Rotate image */
-	case GDK_bracketright:
+		/* Rotate clockwise without saving */
 	case GDK_r: 
-		gth_window_activate_action_alter_image_rotate90 (NULL, viewer);
-		retval = TRUE;
-		break;
-			
+		gth_window_activate_action_alter_image_rotate90 (NULL, window);
+		return TRUE;
+
+		/* Rotate counter-clockwise without saving */
+	case GDK_e:
+		gth_window_activate_action_alter_image_rotate90cc (NULL, window);
+		return TRUE;
+
+		/* Lossless clockwise rotation. */
+	case GDK_bracketright:
+		gth_window_activate_action_tools_jpeg_rotate_right (NULL, window);
+		return TRUE;
+
+		/* Lossless counter-clockwise rotation */
+	case GDK_bracketleft:
+		gth_window_activate_action_tools_jpeg_rotate_left (NULL, window);
+		return TRUE;
+
 		/* Flip image */
 	case GDK_l:
-		gth_window_activate_action_alter_image_flip (NULL, viewer);
-		retval = TRUE;
-		break;
+		gth_window_activate_action_alter_image_flip (NULL, window);
+		return TRUE;
 
 		/* Mirror image */
 	case GDK_m:
-		gth_window_activate_action_alter_image_mirror (NULL, viewer);
-		retval = TRUE;
-		break;
-
-		/* Rotate image counter-clockwise */
-	case GDK_bracketleft:
-		gth_window_activate_action_alter_image_rotate90cc (NULL, viewer);
-		retval = TRUE;
-		break;
+		gth_window_activate_action_alter_image_mirror (NULL, window);
+		return TRUE;
 
 		/* Full screen view. */
 	case GDK_v:
@@ -1863,7 +1870,7 @@ gth_viewer_construct (GthViewer   *viewer,
 	/* * image_vpaned */
 
 	priv->image_main_pane = image_vpaned = gtk_vpaned_new ();
-	gtk_paned_set_position (GTK_PANED (image_vpaned), eel_gconf_get_integer (PREF_UI_COMMENT_PANE_SIZE, DEFAULT_WIN_WIDTH / 3 * 2));
+	gtk_paned_set_position (GTK_PANED (viewer->priv->image_main_pane), eel_gconf_get_integer (PREF_UI_VIEWER_HEIGHT, DEFAULT_WIN_HEIGHT) - eel_gconf_get_integer (PREF_UI_COMMENT_PANE_SIZE, DEFAULT_COMMENT_PANE_SIZE));
 
 	/* ** table */
 
